# Generated by Django 2.1.5 on 2019-03-02 03:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('articles', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='read_only_nullable',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
    ]
