{% load static %}
<!DOCTYPE html>
<html>
<head>
    <title>{{ title }}</title>
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1">

    <link rel="stylesheet" type="text/css" href="{% static 'drf-yasg/style.css' %}"/>
</head>
<body>
    <script id="redoc-settings" type="application/json">{{ redoc_settings | safe }}</script>

    <script src="{% static 'drf-yasg/insQ.min.js' %}"></script>
    <script src="{% static 'drf-yasg/redoc-init.js' %}"> </script>
    <script src="{% static 'drf-yasg/redoc-old/redoc.min.js' %}"></script>
</body>
</html>
