{% load static %}
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{% block title %}{{ title }}{% endblock %}</title>

    {% block extra_head %}
        {# -- Add any extra HTML heads tags here - except scripts and styles -- #}
    {% endblock %}

    {% block favicon %}
        {# -- Maybe replace the favicon -- #}
        <link rel="icon" type="image/png" href="{% static 'drf-yasg/redoc/redoc-logo.png' %}"/>
    {% endblock %}

    {% block main_styles %}
        <link rel="stylesheet" type="text/css" href="{% static 'drf-yasg/style.css' %}"/>
    {% endblock %}
    {% block extra_styles %}
        {# -- Add any additional CSS scripts here -- #}
    {% endblock %}
</head>
<body>

{% block extra_body %}
    {# -- Add any header/body markup here (rendered BEFORE the swagger-ui/redoc element) -- #}
{% endblock %}

<div id="redoc-placeholder"></div>

{% block footer %}
    {# -- Add any footer markup here (rendered AFTER the swagger-ui/redoc element) -- #}
{% endblock %}

<script id="redoc-settings" type="application/json">{{ redoc_settings | safe }}</script>

{% block main_scripts %}
    <script src="{% static 'drf-yasg/insQ.min.js' %}"></script>
    <script src="{% static 'drf-yasg/redoc-init.js' %}"></script>
    <script src="{% static 'drf-yasg/redoc/redoc.min.js' %}"></script>
{% endblock %}
{% block extra_scripts %}
    {# -- Add any additional scripts here -- #}
{% endblock %}
</body>
</html>
