# Generated by Django 2.1 on 2018-08-06 13:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('people', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='identity',
            name='lastName',
            field=models.CharField(help_text="<strong>Here's some HTML!</strong>", max_length=30, null=True),
        ),
        migrations.RenameField(
            model_name='identity',
            old_name='firstName',
            new_name='first_name',
        ),
        migrations.RenameField(
            model_name='identity',
            old_name='lastName',
            new_name='last_name',
        ),
    ]
